import { View, Image, Text } from '@tarojs/components'
import { Switch, Slider } from "@taroify/core"
import ble from "@/assets/ble.png";
// import soundCard from "@/assets/soundCard.png";
import game from "@/assets/game.png";
import lamp from "@/assets/lamp.png";
import Taro, { useDidHide, useDidShow } from '@tarojs/taro';
import { useCallback, useState } from 'react';
import { hex, strInsert, getInitData } from '@/utils/sendOrder';
import { debounce } from '@/utils/index'
import BLESDK from '@/utils/ble'
import PullDownScrollView from "@/components/PullDownScrollView";
import './index.less'

export default function Index() {
  const sourceList = ['Line', 'OPT', 'USB']
  let [ruleForm, setRuleForm] = useState({
    volume: 0, //话筒音量
    music: 0, //音乐音量
    source: 0, //音源设置
    bleSwitch: 0, //蓝牙开关
    soundCadSwitch: 0, //声卡开关
    playMode: 0, //播放模式
    musicMode: 0, //音乐模式
    musicControl: 0, //音乐控制
    girth: 0, //超低音强度
    reverb: 1, //混响模式
    microphoneReverberation: 0, //话筒混响
    AI: 0, //AI
    playPause: 0, //播放暂停
  })
  // let timeout = null
  const musicModeEnum = ['标准', '动感', '轻柔', '安静']
  const reverbEnum = ['KTV', '实力唱将', '朗诵主持', '萌萌卡通', '魔鬼猫', '原声']

  const toGame = () => {
    Taro.navigateTo({ url: '/pages/game/index' })
  }
  const toLampEffect = () => {
    Taro.navigateTo({ url: '/pages/lampEffect/index' })
  }
  const sendCode = useCallback(debounce((e) => {
    BLESDK.writeBleValue(new Uint8Array(strInsert(e)).buffer)
  }), [])
  const refresh = () => {
    // clearTimeout(timeout)
    if (BLESDK.deviceInfo.state) {
      getInitData()
      // timeout = setTimeout(() => {
      //   refresh()
      // }, 3000)
    }
  }

  const handleChange = (e, key) => {
    if (key !== 'nextAndPre') {
      setRuleForm({
        ...ruleForm,
        [key]: e
      })
    }
    let str = ''
    switch (key) {
      case 'volume':
        str = `7B05EA05B2${hex(e)}`
        break;
      case 'music':
        str = `7B05EA06B2${hex(e)}`
        break;
      case 'source':
        str = `7B05EA07B2${hex(e)}`
        break;
      case 'bleSwitch':
        str = `7B05EA08B2${hex(e)}`
        break;
      case 'soundCadSwitch':
        str = `7B05EA09B2${hex(e)}`
        break;
      case 'playMode':
        str = `7B05EA0AB2${hex(e)}`
        break;
      case 'playPause':
        str = `7B05EA0BB2${hex(e ? 1 : 2)}`
        break;
      case 'girth':
        str = `7B05EA0CB2${hex(e)}`
        break;
      case 'reverb':
        str = `7B05EA0DB2${hex(e)}`
        break;
      case 'microphoneReverberation':
        str = `7B05EA0EB2${hex(e)}`
        break;
      case 'AI':
        str = `7B05EA11B2${hex(e)}`
        break;
      case 'musicMode':
        str = `7B05EA0AB2${hex(e)}`
        break;
      case 'nextAndPre':
        if (ruleForm.source == 0) return
        str = `7B05EA0BB2${hex(e)}`
        break;
    }
    sendCode(str)
  }

  const handleMusicMode = (type) => {
    let num = ruleForm.musicMode
    if (type == "+" && ruleForm.musicMode < 3) {
      num++
    } else if (type == "-" && ruleForm.musicMode > 0) {
      num--
    }
    if (num == ruleForm.musicMode) return
    handleChange(num, 'musicMode')
  }

  //混响模式
  const handleReverb = (type) => {
    let num = ruleForm.reverb
    if (type == "+" && ruleForm.reverb < 6) {
      num++
    } else if (type == "-" && ruleForm.reverb > 1) {
      num--
    }
    if (num == ruleForm.reverb) return
    setRuleForm({ ...ruleForm, reverb: num })
    handleChange(num, 'reverb')
  }

  // const control = (type) => {
  //   let num = ruleForm.girth
  //   if (type == "+" && ruleForm.girth < 3) {
  //     num++
  //   } else if (type == "-" && ruleForm.girth > 0) {
  //     num--
  //   }
  //   if (num == ruleForm.girth) return
  //   handleChange(num, 'girth')
  // }
  const BleCallBack = (bytes) => {
    if (bytes[0] == 0x7D && bytes[4] == 0xB0) {
      Taro.showToast({ title: bytes[5] == 0x00 ? '操作失败' : '操作成功', icon: 'none' })
    }

    if (bytes[4] == 0xB3) {
      // 音乐音量(1B)+话筒音量(1B)+音源设置(1B)+蓝牙开关(1B)
      // +声卡开关(1B)+音乐模式(1B)+音乐控制(1B)+超低音强度(1B)+混响模式(1B)+话筒混响(1B)
      setRuleForm(state => {
        //音乐音量
        state.music = bytes[5]
        //话筒音量
        state.volume = bytes[6]
        //音源设置
        state.source = bytes[7]
        //蓝牙开关
        state.bleSwitch = bytes[8]
        //声卡开关
        state.soundCadSwitch = bytes[9]
        //音乐模式
        state.musicMode = bytes[10]
        //音乐控制
        state.playPause = bytes[11]
        //超低音强度
        state.girth = bytes[12]
        //混响模式
        state.reverb = bytes[13]
        //话筒混响
        state.microphoneReverberation = bytes[14]
        return { ...state }
      })
    }
  }

  useDidShow(() => {
    BLESDK.onBLECharacteristicValueChange(BleCallBack)
    refresh()
  })
  useDidHide(() => {
    // clearTimeout(timeout)
  })
  return (
    <PullDownScrollView
      className='PullDownScrollView'
      scrollY
      refresherEnabled
      refresherBackground='#f3f8f7'
      refreshFetcher={refresh}
    >
      <View className='index'>
        <View className='headbox'>
          <View className='source'>
            <View className='title'>音乐信号(音源)</View>
            <View className='list'>
              {
                sourceList.map((item, index) => {
                  return <View className={`list-item ${index == ruleForm.source ? 'active' : ''}`} key={index} onClick={() => handleChange(index, 'source')}>
                    <View className='yuan'></View>
                    <View className='line'></View>
                    <View className='text'>{item}</View>
                  </View>
                })
              }
            </View>
          </View>

          <View className='switch'>
            <View className='title'>开关状态</View>
            <View className='switch-item'>
              <Image className='img' src={ble} />
              <Text className='text'>蓝牙</Text>
              <Switch size='20' onChange={(e) => handleChange(e, 'bleSwitch')} checked={ruleForm.bleSwitch} />
            </View>
            {/* <View className='switch-item'>
            <Image className='img' src={soundCard} />
            <Text className='text'>声卡</Text>
            <Switch size='20' onChange={(e) => handleChange(e, 'soundCadSwitch')} checked={ruleForm.soundCadSwitch} />
          </View> */}
          </View>
        </View>
        <View className='play'>
          <View className={`play-icon ${ruleForm.source == 0 ? 'pre_def' : 'pre'}`} onClick={() => handleChange(3, 'nextAndPre')}></View>
          <View className={ruleForm.playPause ? 'play-pause' : 'play-play'} onClick={() => {
            handleChange(!ruleForm.playPause, 'playPause')
          }}
          ></View>
          <View className={`play-icon ${ruleForm.source == 0 ? 'next_def' : 'next'}`} onClick={() => handleChange(4, 'nextAndPre')}></View>
        </View>
        <View className='card'>
          <View className='funitem'>
            <View className='funitem-text'>音乐模式</View>
            <View className='mode'>
              <View className='icon l' onClick={() => handleMusicMode("-")}></View>
              <View className='input'>{musicModeEnum[ruleForm.musicMode]}</View>
              <View className='icon r' onClick={() => handleMusicMode("+")}></View>
            </View>
          </View>
        </View>
        {/* <View className='card'>
        <View className='funitem'>
          <View className='funitem-text'>超低音强度</View>
          <View className='mode'>
            <View className='icon l' onClick={() => control("-")}></View>
            <View className='input'>{ruleForm.girth}</View>
            <View className='icon r' onClick={() => control("+")}></View>
          </View>
        </View>
      </View> */}
        <View className='card'>
          <View className='funitem'>
            <View className='funitem-text'>音乐音量</View>
            <View className='funitem-right'>
              <View className='num'>{ruleForm.volume}%</View>
              <Slider className='custom-color' min={0} max={100} size={4} onChange={(e) => handleChange(e, 'volume')} value={ruleForm.volume} >
                <Slider.Thumb>
                  <View className='custom-thumb'></View>
                </Slider.Thumb>
              </Slider>
            </View>
          </View>
        </View>
        <View className='card'>
          <View className='funitem'>
            <View className='funitem-text'>话筒音量</View>
            <View className='funitem-right'>
              <View className='num'>{ruleForm.music}%</View>
              <Slider className='custom-color' min={0} max={100} size={4} onChange={(e) => handleChange(e, 'music')} value={ruleForm.music} >
                <Slider.Thumb>
                  <View className='custom-thumb'></View>
                </Slider.Thumb>
              </Slider>
            </View>
          </View>
          {/* <View className='funitem'>
          <View className='funitem-text'>话筒混响音量</View>
          <View className='funitem-right'>
            <View className='num'>{ruleForm.microphoneReverberation}%</View>
            <Slider className='custom-color' min={0} max={32} size={4} onChange={(e) => handleChange(e, 'microphoneReverberation')} value={ruleForm.microphoneReverberation} >
              <Slider.Thumb>
                <View className='custom-thumb'></View>
              </Slider.Thumb>
            </Slider>
          </View>
        </View> */}
          <View className='funitem'>
            <View className='funitem-text'>话筒混响模式</View>
            <View className='mode'>
              <View className='icon l' onClick={() => handleReverb("-")}></View>
              <View className='input'>{reverbEnum[ruleForm.reverb - 1]}</View>
              <View className='icon r' onClick={() => handleReverb("+")}></View>
            </View>
            {/* <View className='numlist'>
              {[1, 2, 3, 4, 5, 6].map(item => {
                return <View className={`numlist-num ${ruleForm.reverb == item ? 'active' : ''} `} key={item} onClick={() => {
                  if (item == ruleForm.reverb) return
                  handleChange(item, 'reverb')
                }}
                >{item}</View>
              })}
            </View> */}
          </View>
        </View>
        <View className='btns'>
          <View className='btn-item' onClick={toGame}>
            <Image src={game} className='img' />
            GAME
          </View>
          <View className='btn-item' onClick={toLampEffect}>
            <Image src={lamp} className='img' />
            灯效
          </View>
        </View>
      </View>
    </PullDownScrollView>

  )
}
